import io
import os
import flask
import requests
from jinja2 import Template


def _read_if_exists(url_prefix, custom_prefix, suffix, engine):

    if url_prefix is None:
        custom_file_name = custom_prefix + "." + suffix
        if os.path.isfile(custom_file_name):
            with io.open(custom_file_name, "r") as f:
                return f.read()
    else:
        url = url_prefix + "/" + custom_prefix + "." + suffix
        response = requests.get(url)
        if response.status_code != 404:
            return response.text

    vendor_file_name = os.path.join(
        os.path.dirname(__file__), "static", "engines", engine, "vendor." + suffix
    )
    if os.path.isfile(vendor_file_name):
        with io.open(vendor_file_name, "r") as f:
            return f.read()

    return ""


def render(engine, url_prefix, md_file_prefix, markdown):

    engine_root = flask.url_for("static", filename="engines/" + engine)

    # flask.Markup to disable autoescaping
    custom_css = flask.Markup(
        _read_if_exists(url_prefix, md_file_prefix, "css", engine)
    )

    _tmp = _read_if_exists(url_prefix, md_file_prefix, "head.html", engine)
    custom_head_html = flask.Markup(Template(_tmp).render(engine_root=engine_root))

    _tmp = _read_if_exists(url_prefix, md_file_prefix, "body.html", engine)
    custom_body_html = flask.Markup(
        Template(_tmp).render(markdown=markdown, engine_root=engine_root)
    )

    return flask.render_template(
        "render.html",
        title="presentation",
        custom_css=custom_css,
        custom_head_html=custom_head_html,
        custom_body_html=custom_body_html,
        engine=engine,
    )



import re


def expand_img_link(s, prefix):

    if "![" in s and "](" in s:
        if "http" not in s:
            p = re.compile(r"!\[(.*)\]\(")
            alt = p.findall(s)[0]
            p = re.compile(r'!\[.*\]\(([^")]+)')
            src = p.findall(s)[0]
            return "![{0}]({1}{2})\n".format(alt, prefix, src)

    if "<img" in s and 'src="' in s:
        if "http" not in s:
            p = re.compile(r'<img [^>]*src="([^"]+)')
            src = p.findall(s)[0]
            return s.replace(src, prefix + src)

    if "<source" in s and 'src="' in s:
        if "http" not in s:
            p = re.compile(r'<source [^>]*src="([^"]+)')
            src = p.findall(s)[0]
            return s.replace(src, prefix + src)

    if "background-image: url(" in s:
        if "http" not in s:
            p = re.compile(r'background-image: url\(([^")]+)')
            src = p.findall(s)[0]
            return s.replace(src, prefix + src)

    return s


def test_expand_img_link():
    assert (
        expand_img_link("![Raboof](img/pie.jpg)", "foo/")
        == "![Raboof](foo/img/pie.jpg)\n"
    )
    assert (
        expand_img_link("![Foo oof 123](img/pie.jpg)", "http://")
        == "![Foo oof 123](http://img/pie.jpg)\n"
    )
    assert expand_img_link("[Raboof](img/pie.jpg)", "foo/") == "[Raboof](img/pie.jpg)"
    assert expand_img_link("img src", "foo/") == "img src"
    assert (
        expand_img_link('<img src="img/phd_final.gif" style="width: 400px;"/>', "foo/")
        == '<img src="foo/img/phd_final.gif" style="width: 400px;"/>'
    )
    assert (
        expand_img_link('<source src="img/cat.mp4" type="video/mp4">', "foo/")
        == '<source src="foo/img/cat.mp4" type="video/mp4">'
    )
    assert (
        expand_img_link("background-image: url(foo.png)", "bar/")
        == "background-image: url(bar/foo.png)"
    )


def fix_images(s, prefix):
    return "\n".join([expand_img_link(line, prefix) for line in s.split("\n")])



import io
import os
import flask
import sys
import requests
import json
from .version import __version__
from .images import fix_images
from .render import render

blueprint = flask.Blueprint("git", __name__)

URL_BASE = "CICERO_URL_BASE_is_undefined"
_url_base = os.environ.get("CICERO_URL_BASE")
if _url_base is not None:
    URL_BASE = _url_base


def get_sha_github(owner, repo, ref):
    uri = "https://api.github.com/repos/{0}/{1}/commits/{2}".format(owner, repo, ref)
    response = requests.get(uri)
    data = json.loads(response.text)
    return data["sha"]


def test_get_sha_github():
    sha = get_sha_github("bast", "cicero", "bfa3748447")
    assert sha == "bfa3748447fe0c7455f19a027575406a0c561a4f"


def set_url_base(host, port):
    global URL_BASE
    URL_BASE = "http://{}:{}".format(host, port)


@blueprint.route("/")
def home():
    return flask.render_template("index.html", url_base=URL_BASE, version=__version__)


def render_url_markdown(path, engine, engine_version):

    service, owner, repo, ref, *_md_file_path = path.split("/")
    md_file_path = "/".join(_md_file_path)
    md_file_path_root = "/".join(_md_file_path[:-1])
    md_file = _md_file_path[-1]

    if service == "github.com":
        # in the old days we used cdn.rawgit.com which cached files forever
        # and therefore instead of referencing the branch, we referenced the sha
        # not sure whether raw.githubusercontent.com caches files
        # so translating to hashes might be overkill
        sha = get_sha_github(owner, repo, ref)
        url_prefix = "https://raw.githubusercontent.com/{0}/{1}/{2}/".format(
            owner, repo, sha
        )
        # we use this image_url_prefix workaround since
        # raw.githubusercontent.com does not render svg
        image_url_prefix = "https://cdn.jsdelivr.net/gh/{0}/{1}@{2}".format(
            owner, repo, sha
        )
    elif service == "gitlab.com":
        url_prefix = "https://{0}/{1}/{2}/raw/{3}/".format(service, owner, repo, ref)
        image_url_prefix = url_prefix
    else:
        *_url_prefix, md_file = path.split("/")
        url_prefix = "https://" + "/".join(_url_prefix)
        image_url_prefix = url_prefix
        md_file_path = md_file
        md_file_path_root = ""

    md_file_prefix, _ = os.path.splitext(md_file)

    if "/" in md_file_path:
        url_prefix += md_file_path_root

    url = url_prefix + "/" + md_file
    response = requests.get(url)
    if response.status_code == 404:
        return flask.render_template("404.html")
    markdown = response.text

    markdown = fix_images(markdown, image_url_prefix + "/")

    return render(
        engine="{0}-{1}".format(engine, engine_version),
        url_prefix=url_prefix,
        md_file_prefix=md_file_prefix,
        markdown=markdown,
    )


@blueprint.route("/v1/github/<path:path>/remark/")
def render_v1(path):
    return render_url_markdown("github.com" + "/" + path, "remark", "legacy")


@blueprint.route("/v2/remark/github/<path:path>/")
def render_v2(path):
    return render_url_markdown("github.com" + "/" + path, "remark", "legacy")


@blueprint.route("/v3/<string:engine>/<string:engine_version>/<path:path>/")
def render_v3(path, engine, engine_version):
    return render_url_markdown(path, engine, engine_version)



import io
import os
import flask
from .images import fix_images
from .render import render

blueprint = flask.Blueprint("preview", __name__)


@blueprint.route("/")
def home():
    config = flask.current_app.config

    md_file_prefix, _ = os.path.splitext(config["filename"])

    try:
        with io.open(config["filename"], "r", encoding="utf-8") as f:
            markdown = f.read()
    except UnicodeDecodeError:
        with io.open(config["filename"], "r", encoding="cp1252") as f:
            markdown = f.read()
    markdown = fix_images(markdown, "images/")

    return render(
        engine=config["engine"],
        url_prefix=None,
        md_file_prefix=md_file_prefix,
        markdown=markdown,
    )


@blueprint.route("/images/<path:path>")
def serve_image(path):
    config = flask.current_app.config
    return flask.send_from_directory(config["imagedir"], path)


@blueprint.errorhandler(404)
def page_not_found(e):
    return flask.render_template("404.html"), 404



from argparse import ArgumentParser, ArgumentDefaultsHelpFormatter
import os
import glob
import ntpath


def _get_list_of_engines():
    engines = []
    for path in glob.glob(
        os.path.join(os.path.dirname(__file__), "static", "engines", "*")
    ):
        engines.append(ntpath.basename(path))
    return sorted(engines)


def parse_args():
    parser = ArgumentParser(formatter_class=ArgumentDefaultsHelpFormatter)
    arg = parser.add_argument
    engines = ", ".join(_get_list_of_engines())

    arg("--file", "-f", dest="filename", help="serve a local file")
    arg(
        "--engine",
        dest="engine",
        help="rendering engine (available: {0})".format(engines),
        default="remark-0.14.0",
    )
    arg("--debug", dest="debug", action="store_true", default=False)
    arg("--host", dest="host", default=os.environ.get("HOST", "127.0.0.1"))
    arg("--port", dest="port", type=int, default=int(os.environ.get("PORT", 5000)))

    return parser.parse_args()



from flask import Flask, render_template
import os


def _get_subdir(dirname):
    return os.path.join(os.path.dirname(__file__), dirname)


app = Flask(
    "Cicero",
    template_folder=_get_subdir("templates"),
    static_folder=_get_subdir("static"),
)


@app.errorhandler(404)
def page_not_found(e):
    return render_template("404.html"), 404
